import fetch from 'node-fetch'; // Ensure this is the correct version
import { Buffer } from 'buffer'; // Use direct import

// Define an interface for the expected response from the Spotify API
interface SpotifyTokenResponse {
    access_token: string;
    token_type: string;
    expires_in: number;
}

// Type guard to check if the object matches SpotifyTokenResponse
function isSpotifyTokenResponse(data: any): data is SpotifyTokenResponse {
    return (
        typeof data === 'object' &&
        data !== null &&
        'access_token' in data &&
        'token_type' in data &&
        'expires_in' in data
    );
}

export async function getAccessToken(): Promise<string> {
    const clientId = process.env.SPOTIFY_CLIENT_ID;
    const clientSecret = process.env.SPOTIFY_CLIENT_SECRET;

    if (!clientId || !clientSecret) {
        throw new Error("Missing Spotify credentials");
    }

    const response = await fetch("https://accounts.spotify.com/api/token", {
        method: "POST",
        headers: {
            "Content-Type": "application/x-www-form-urlencoded",
            Authorization: "Basic " + Buffer.from(`${clientId}:${clientSecret}`).toString("base64"),
        },
        body: new URLSearchParams({
            grant_type: "client_credentials",
        }),
    });

    if (!response.ok) {
        throw new Error("Failed to fetch access token");
    }

    const data: unknown = await response.json(); // Keep as unknown first

    // Use the type guard to check the shape of the data
    if (!isSpotifyTokenResponse(data)) {
        throw new Error("Invalid response structure from Spotify API");
    }

    // Now TypeScript knows data is of type SpotifyTokenResponse
    return data.access_token; 
}
